// detect.c by Markus Stark

#define _DETECT_C_

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <assert.h>
#include <sys/times.h>
#include <time.h>

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h> 

#include "detect.h"

#define DEBUGVALUES

int read_intvar(char *line, char *name, int *value)
{
  int j;
  
  if ((strncmp(line, name, strlen(name)))==0)
  {
  	
	for (j=strlen(name); j<80; j++)
	{
			if (line[j]=='=')
			{
				if (sscanf(&line[j+1],"%d",value))
				{
				#ifdef DEBUGVALUES
					printf("%s set to %d\n",name,*value);
				#endif
					return 1;
				}
				else
				{
					fprintf(stderr, "Error reading config file: >%s< ?",line);
					exit (-1);
				}
			}
	}
	fprintf(stderr, "Error in config file: >%s<\n",name);
	return 0;
  }
  else
  {
  	return 0;
  }
}

int read_doublevar(char *line, char *name, double *value)
{
  int j;
   
  if ((strncmp(line, name, strlen(name)))==0)
  {
  
	for (j=strlen(name); j<80; j++)
	{
			if (line[j]=='=')
			{
				if ((sscanf(&line[j+1],"%lf",value))==1)
				{
				#ifdef DEBUGVALUES
				        printf("Line: >%s<\n",line);
					printf("%s set to %*value\n",name,*value);
				#endif
					return 1;
				}
				else
				{
					fprintf(stderr, "Error reading config file: >%s< ?",line);
					exit (-1);
				}
			}
	}
	fprintf(stderr, "Error in config file: >%s<\n",name);
	return 0;
  }
  else
  {
  	return 0;
  }
}

int DetectInit (int argc, char **argv)
{
  FILE *fp;
  char line[100];
  struct tms zeitbuf;                 
  
  
  // Sicherheitshalber Defaultwerte setzen
  DetectVar.sensitivity = 7;
  DetectVar.calibrate = 0;
  DetectVar.mintime = 3.0;
  DetectVar.beginpause = 5.0;
  DetectVar.prgstarttime= times(&zeitbuf) / 100.0;
  DetectVar.highcolor = 0 ;  // 0= 256 Farben, 1=24 Bit Farben
  DetectVar.calibrate = 0;  // Kalibrierbmodus
  DetectVar.videowdth = 176; // bisher unbenutzt
  DetectVar.videohgth = 144; 
  
  DetectVar.xmin[0] = 20;  // 1. Fenster das untersucht wird
  DetectVar.xmax[0] = 156;
  DetectVar.ymin[0] = 58;
  DetectVar.ymax[0] = 78;
  
  DetectVar.xmin[1] = 20;  // 2. Fenster das untersucht wird
  DetectVar.xmax[1] = 156;
  DetectVar.ymin[1] = 98;
  DetectVar.ymax[1] = 118;
  
  DetectVar.xstep = 4;
  DetectVar.ystep = 2;
  DetectVar.camdistance = 1.0;
  DetectVar.brightness=DetectVar.contrast=DetectVar.saturation=DetectVar.hue =
    65535 / 2;
  
   
  // Konfigurationsdatei oeffnen
  fp = fopen ("vilap.cnf","r");
  if (fp == NULL)
  {

	fprintf (stderr, "no vilap.cnf configuration file found. Using defaults\n");
  }
  else
  {
  	while (!feof(fp))
	{
     	 if (fgets(line, 80, fp)!=NULL){
	 	if (!(line[0]=='#')){
    	 	if (!read_intvar   (line,"SENSITIVITY", &DetectVar.sensitivity)){
    	 	if (!read_doublevar(line,"MINLAPTIME", &DetectVar.mintime)){
		if (!read_doublevar(line,"BEGINPAUSE", &DetectVar.beginpause)){
		if (!read_doublevar(line,"CAMDISTANCE", &DetectVar.camdistance)){
		if (!read_intvar   (line,"HIGHCOLOR", &DetectVar.highcolor)){
		if (!read_intvar   (line,"XMIN1", &DetectVar.xmin[0])){
		if (!read_intvar   (line,"YMIN1", &DetectVar.ymin[0])){
		if (!read_intvar   (line,"XMAX1", &DetectVar.xmax[0])){
		if (!read_intvar   (line,"YMAX1", &DetectVar.ymax[0])){
		if (!read_intvar   (line,"XMIN2", &DetectVar.xmin[1])){
		if (!read_intvar   (line,"YMIN2", &DetectVar.ymin[1])){
		if (!read_intvar   (line,"XMAX2", &DetectVar.xmax[1])){
		if (!read_intvar   (line,"YMAX2", &DetectVar.ymax[1])){
		if (!read_intvar   (line,"XSTEP", &DetectVar.xstep)){
		if (!read_intvar   (line,"YSTEP", &DetectVar.ystep)){
		if (!read_intvar   (line,"BRIGHTNESS", &DetectVar.brightness)){
		if (!read_intvar   (line,"CONTRAST", &DetectVar.contrast)){
		if (!read_intvar   (line,"SATURATION", &DetectVar.saturation)){
		if (!read_intvar   (line,"HUE", &DetectVar.hue))
		{
			if (line[0]!='\n')
			{
				fprintf (stderr,"Warning: wrong Identifier >%s< in config file.\n",line);
			}
		}}}}}}}}}}}}}}}}}}}}
		
	 }
  	}
		
  }
  
  // Argumentverarbeitung
  if (argc>1)
  {
    if (strcmp(argv[1], "-c1")==0)
    {
    	DetectVar.calibrate = 1; 
	puts("Calibrate Mode for car 1");
    }
    else if (strcmp(argv[1], "-c2")==0)
    {
	DetectVar.calibrate = 2;
	puts("Calibrate Mode for car 2");
    }
  }
  
  
  printf("\nGentlemen, Start your engines!\n\n%2.0f seconds till race will start\n\n\n", DetectVar.beginpause);
  
  return 1;
} // end DetectInit

#define noTDEBUG
#define noDEBUG

void DetectCar (XImage *xImage)
{
	int i;         // Zhlvariable
	clock_t zeit;  // aktuelle Zeit
	struct tms zeitbuf; // Zeit beim vorigen berqueren der Zielline
	double	      dzeit; 	  // aktuelle Zeit
        unsigned long farbe;          // Farbe des aktuell untersuchten Bildpunktes
	unsigned long r=0,g=0,b=0; // rot, grn, blau - Anteil
	int x,y;  // x,y Koordinate des zu untersuchenden Punktes
	static double bestlap=999999999.0, rundenzeit;
	static int kannlosgehen=1;	
	int car;

zeit = times(&zeitbuf);
dzeit = zeit / 100.0;

#ifdef TDEBUG
printf ("%f ",dzeit - dzeitalt);
#endif

// Rennen wird nicht sofort gestartet!
if ((dzeit-DetectVar.prgstarttime)<DetectVar.beginpause)  
  return;
if (kannlosgehen)
{
	printf("\n>>>>>>>>> GO <<<<<<<<<<\n\n");fflush(stdout);
	kannlosgehen=0;
	dzeit=0.0;
	DetectVar.starttime= zeit / 100.0;
	
	for (car=0; car<NUMCARS; car++)
	{
		DetectVar.runde[car]=0;
		DetectVar.dzeitalt[car]= DetectVar.starttime;
	}
}

for (car=0; car<NUMCARS; car++)
{	
for (y=DetectVar.ymin[car]; y<DetectVar.ymax[car]; y+=DetectVar.ystep) 
{ 
for (x=DetectVar.xmin[car]; x<DetectVar.xmax[car]; x+=DetectVar.xstep)  // Doppelte Schleife
{
	farbe=XGetPixel(xImage, x, y);
	
	if (DetectVar.highcolor)  
	{
		b= (farbe & 0xFF) / 0x10 ;
		g= (farbe & 0xFF00) / 0x100;  
		r= (farbe & 0xFF0000) / 0x10000;
	}
	else
	{
		b = farbe & 0xF;
		g = (farbe & 0xF0) / 0x10;
		r = (farbe & 0xF00) / 0x100;
	}

	if (DetectVar.calibrate)
	{
	   if ((DetectVar.calibrate-1)==car)
	   {
		printf ("%x%x%x ", r,g,b); 
		if (x>=(DetectVar.xmax[car]-DetectVar.xstep))
		{
	  		printf("\n");
			fflush (stdout);
		}
	   }
	}
	else 
	{
	if ((r<DetectVar.sensitivity) || (g<DetectVar.sensitivity) || (b<DetectVar.sensitivity))  // Kontakt!
	{   
#ifdef DEBUG
	   printf (">>>>sens:%d, f:%x %x %x %x x,y=%d,%d lap=%d, car=%d\n", 
	     DetectVar.sensitivity, farbe, r,g,b,x,y, runde, car+1); fflush (stdout);
#endif

		if (DetectVar.runde[car]==0)  // START
		{
			dzeit = zeit / 100.0;
			DetectVar.runde[car]=1;
			DetectVar.dzeitalt[car]=dzeit;
			printf("Car %d is started!\n",car+1);fflush(stdout);
		}
		else  //Runde im Rennen
		{	
			dzeit= zeit / 100.0;
			if (dzeit-DetectVar.dzeitalt[car] >= DetectVar.mintime)
			{
				rundenzeit = dzeit-DetectVar.dzeitalt[car];
				if (rundenzeit<bestlap)
				{
					bestlap = rundenzeit;
					printf(",,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,\n");
					printf(">>>>>>> New best lap:  %6.2f <<<<<<<\n\b",bestlap);
					printf("'''''''''''''''''''''''''''''''''''''\n");
				}
				printf("Car:%d Lap:%4d Total:%8.2fs Last:%6.2fs\n", 
				   car+1, DetectVar.runde[car], dzeit-DetectVar.starttime, rundenzeit);
				fflush(stdout);
				DetectVar.dzeitalt[car] = dzeit;
				DetectVar.runde[car] = DetectVar.runde[car]+1;
			}
			else
			{
				#ifdef DEBUG
				puts ("Kleiner Mintime");
				#endif
			}

		}
        }
	//else  // Kein Kontakt
	//{
	//   Z.Z. nix zu tun
	//}
	}	
}  // end for y
}  // end for x
}  // end for car
return ;

}  // end DetectCar()



